//BTN1 RANDOM NOTE AND VELOCITY
//BTN2 SEQ_PAGE NOTE AND VELOCITY/TRANSPOSE_LENGTH PAGE
//BTN3 STEP_UP_SEQ
//BTN4 STEP_DOWN_SEQ



#include <MidiTools.h>

#include "WProgram.h"
void randomizeNotes();
void on16Callback();
void setup();
void loop();
void handleGui();
void updateSeqGui();
RangeEncoder lengthEncoder(1, 16, "LEN");
RangeEncoder transposeEncoder(0, 4, "TRN");
RangeEncoder noteAtCurrentStep(0, 127, "PCH");
RangeEncoder velocityAtCurrentStep(0, 127, "VEL");
EncoderPage page(&lengthEncoder,&transposeEncoder);
EncoderPage page2(&velocityAtCurrentStep,&noteAtCurrentStep);
uint8_t melodyNotes[16] = { 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48,48,48,48,48,48 };
uint8_t velocityNotes[16] = { 127, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
uint8_t transposeNotes[5] = {0, 3, 5, 7, 12};
uint8_t prevNote = 0;
uint8_t currentStep = 0;
uint8_t length = 8;
uint8_t togglePage = 0;


void randomizeNotes() {
	for (int i = 0; i < countof(melodyNotes); i++) {
		melodyNotes[i] = scalePitch(48 + random(24), 0, minorScale);
                velocityNotes[i] = (random(5)*32)%128;
	}
}


void on16Callback() {
  
	MidiUart.sendNoteOff(prevNote);
        if((MidiClock.div16th_counter%16)==0) length=lengthEncoder.getValue();
	prevNote = melodyNotes[MidiClock.div16th_counter % length]+transposeNotes[transposeEncoder.getValue()];
	MidiUart.sendNoteOn(prevNote,  velocityNotes[MidiClock.div16th_counter % length]);
}



void setup() {
	//randomizeNotes();
	lengthEncoder.setValue(8);
	MidiClock.mode = MidiClock.INTERNAL;
	MidiClock.setTempo(100);
	MidiClock.setOn16Callback(on16Callback);
	MidiClock.start();
        GUI.setPage(&page);
        updateSeqGui();
}

void loop() {
	GUI.updatePage();
        if(noteAtCurrentStep.hasChanged()){
          melodyNotes[currentStep] = noteAtCurrentStep.getValue();
        }
        if(velocityAtCurrentStep.hasChanged()){
          velocityNotes[currentStep] = velocityAtCurrentStep.getValue();
        }

        GUI.setLine(GUI.LINE1);
        GUI.put_string_at(12,"STP");
        GUI.setLine(GUI.LINE2);
        GUI.put_value_at(12,currentStep);
	GUI.update();
}

void handleGui() {
	if (BUTTON_PRESSED(Buttons.BUTTON1)) {
		randomizeNotes();
                updateSeqGui();
	}
	if (BUTTON_PRESSED(Buttons.BUTTON4)&&(togglePage == 1)) {
                if(currentStep<15) currentStep++;
                updateSeqGui();       
	}
	if (BUTTON_PRESSED(Buttons.BUTTON3)&&(togglePage == 1)) {
		if(currentStep>0) currentStep--;
                updateSeqGui();
	}
	if (BUTTON_PRESSED(Buttons.BUTTON2)) {
	togglePage = 1 - togglePage;
        if(togglePage == 1) {
          GUI.setPage(&page2);
        } else {
          GUI.setPage(&page);
        }
	}        
}

void updateSeqGui() {
    noteAtCurrentStep.setValue(melodyNotes[currentStep]);
    velocityAtCurrentStep.setValue(velocityNotes[currentStep]);
}

#include <Midi.h>

#include "WProgram.h"
#include <avr/interrupt.h>

void setup();
void loop();
void handleGui();

#define PHASE_FACTOR 16
static inline uint32_t phase_mult(uint32_t val) {
  return (val * PHASE_FACTOR) >> 8;
}

ISR(TIMER1_OVF_vect) {
  //  setLed2();

  clock++;
#ifdef MIDIDUINO_MIDI_CLOCK
  if (MidiClock.state == MidiClock.STARTED)
    MidiClock.handleTimerInt();
#endif

  //  clearLed2();
}

// XXX CMP to have better time

void gui_poll() {
  uint16_t sr = SR165.read16();
  Buttons.clear();
  Buttons.poll(sr >> 8);
  Encoders.poll(sr);
}

uint16_t lastRunningStatusReset = 0;

ISR(TIMER2_OVF_vect) {
  //  setLed2();

#ifdef MIDIDUINO_POLL_GUI_IRQ
  gui_poll();
  handleGui();
#endif
  slowclock++;

  if (abs(slowclock - lastRunningStatusReset) > 3000) {
    MidiUart.resetRunningStatus();
    lastRunningStatusReset = slowclock;
  }
  
  //  clearLed2();
}

MidiClass Midi;
MidiClass Midi2;

void __mainInnerLoop(bool callLoop) {
    if (MidiUart.avail()) {
      Midi.handleByte(MidiUart.getc());
    }

    if (MidiUart2.avail()) {
      Midi2.handleByte(MidiUart2.getc());
    }
    
#if defined(MIDIDUINO_POLL_GUI) && !defined(MIDIDUINO_POLL_GUI_IRQ)
    cli();
    gui_poll();
    handleGui();
    sei();
#endif

    if (callLoop)
      loop();
}

int main(void) {
    delay(100);
  init();
  clearLed();
  clearLed2();

  for (int i = 0; i < GUI_NUM_BUTTONS; i++) {
    SET_B_CURRENT(i);
  }

  MidiSysex.addSysexListener(&MididuinoSysexListener);
  setup();
  sei();

  for (;;) {
    __mainInnerLoop();
  }
  return 0;
}
