//BTN1 RANDOM NOTE AND VELOCITY
//BTN2 SEQ_PAGE NOTE AND VELOCITY/TRANSPOSE_LENGTH PAGE
//BTN3 STEP_UP_SEQ
//BTN4 STEP_DOWN_SEQ



#include <MidiTools.h>

RangeEncoder lengthEncoder(1, 16, "LEN");
RangeEncoder transposeEncoder(0, 4, "TRN");
RangeEncoder noteAtCurrentStep(0, 127, "PCH");
RangeEncoder velocityAtCurrentStep(0, 127, "VEL");
EncoderPage page(&lengthEncoder,&transposeEncoder);
EncoderPage page2(&velocityAtCurrentStep,&noteAtCurrentStep);
uint8_t melodyNotes[16] = { 48, 48, 48, 48, 48, 48, 48, 48, 48, 48, 48,48,48,48,48,48 };
uint8_t velocityNotes[16] = { 127, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
uint8_t transposeNotes[5] = {0, 3, 5, 7, 12};
uint8_t prevNote = 0;
uint8_t currentStep = 0;
uint8_t length = 8;
uint8_t togglePage = 0;


void randomizeNotes() {
	for (int i = 0; i < countof(melodyNotes); i++) {
		melodyNotes[i] = scalePitch(48 + random(24), 0, minorScale);
                velocityNotes[i] = (random(5)*32)%128;
	}
}


void on16Callback() {
  
	MidiUart.sendNoteOff(prevNote);
        if((MidiClock.div16th_counter%16)==0) length=lengthEncoder.getValue();
	prevNote = melodyNotes[MidiClock.div16th_counter % length]+transposeNotes[transposeEncoder.getValue()];
	MidiUart.sendNoteOn(prevNote,  velocityNotes[MidiClock.div16th_counter % length]);
}



void setup() {
	//randomizeNotes();
	lengthEncoder.setValue(8);
	MidiClock.mode = MidiClock.INTERNAL;
	MidiClock.setTempo(100);
	MidiClock.setOn16Callback(on16Callback);
	MidiClock.start();
        GUI.setPage(&page);
        updateSeqGui();
}

void loop() {
	GUI.updatePage();
        if(noteAtCurrentStep.hasChanged()){
          melodyNotes[currentStep] = noteAtCurrentStep.getValue();
        }
        if(velocityAtCurrentStep.hasChanged()){
          velocityNotes[currentStep] = velocityAtCurrentStep.getValue();
        }

        GUI.setLine(GUI.LINE1);
        GUI.put_string_at(12,"STP");
        GUI.setLine(GUI.LINE2);
        GUI.put_value_at(12,currentStep);
	GUI.update();
}

void handleGui() {
	if (BUTTON_PRESSED(Buttons.BUTTON1)) {
		randomizeNotes();
                updateSeqGui();
	}
	if (BUTTON_PRESSED(Buttons.BUTTON4)&&(togglePage == 1)) {
                if(currentStep<15) currentStep++;
                updateSeqGui();       
	}
	if (BUTTON_PRESSED(Buttons.BUTTON3)&&(togglePage == 1)) {
		if(currentStep>0) currentStep--;
                updateSeqGui();
	}
	if (BUTTON_PRESSED(Buttons.BUTTON2)) {
	togglePage = 1 - togglePage;
        if(togglePage == 1) {
          GUI.setPage(&page2);
        } else {
          GUI.setPage(&page);
        }
	}        
}

void updateSeqGui() {
    noteAtCurrentStep.setValue(melodyNotes[currentStep]);
    velocityAtCurrentStep.setValue(velocityNotes[currentStep]);
}
